#!/usr/bin/env python
# coding: utf-8
# """
# @Author: JokerMa
# @Eamil: mazhiguo@kylinos.com.cn
# @Time: 2020.12
# @Instruction: Provide class SystemRepair for repair.
# """

import json
import os
import gettext
import time

from settings import CUSTOM_ADD_LOG, CUSTOM_DEL_LOG, CUSTOM_VERSION_DIFFER, RPM_INFO_JSONF, CHROOT_PATH
from utils import quit_judge, yes_or_no, progress
from logutils import report
from oscmd import get_rpm_info, get_rpm_list, call_cmd


# i18n
gettext.install("kylinrepair", "/usr/share/locale")


class SystemRepair(object):
    """ repair system """

    def __init__(self, root_path=""):
        if root_path:
            self.root_path = root_path
        else:
            self.root_path = CHROOT_PATH

        self.cur_rpm_dict = {}
        self.ori_rpm_dict = {}

        # save to set
        self.set_common = set()

        self.set_custom_add = set()
        self.set_custom_del = set()

        self.set_common_to_upgrade = set()
        self.set_common_to_downgrade = set()
        self.set_common_to_reinstall = set()

        retDict = call_cmd("arch")
        self.arch = retDict['stdout'].decode("utf-8").replace("\n", "")

    def info_diff(self, opt="check"):
        """ read the json file and the RPM list infomation of current system """
        print(_("start reading package from current system ..."))
        self.cur_rpm_info()
        print(_("\nread packages info from current system finished!"))

        print(_("start reading package from origin system(json) ..."))
        with open(self.root_path + RPM_INFO_JSONF, "r") as f:
            content = f.read()
        self.ori_rpm_dict = dict(json.loads(content))
        # print(self.ori_rpm_dict)
        print(_("\nread packages info from origin system(json) finished!"))
        self.dict_set_compare()

        # handle
        if opt == "handle":
            if self.set_custom_del:
                self.handle_del()
            if self.set_common_to_upgrade ^ self.set_common_to_downgrade:
                self.handle_changed()
            if self.set_custom_add:
                self.handl_add()

        # handle_all
        if opt == "handle_all":
            if self.set_custom_del:
                self.handle_del(a_tag="hd_all")
            if self.set_common_to_upgrade ^ self.set_common_to_downgrade:
                self.handle_changed(a_tag="hd_all")
            # if self.set_custom_add:
            #     self.handl_add(a_tag="hd_all")
            if self.set_common_to_reinstall:
                self.hadle_rest()

    def fix_some_pkgs(self):
        pkg_prefix = "/mnt/Packages"

        pkg_vrlzma = "pylibzma-*" + self.arch + "*.rpm"
        pkg_bindex = "bind-export-libs-*" + self.arch + "*.rpm"
        pkg_chrony = "chrony-*" + self.arch + "*.rpm"

        cmd = "dnf reinstall %s/%s %s/%s %s/%s -y --installroot=%s" % (pkg_prefix, pkg_vrlzma, pkg_prefix, pkg_bindex, pkg_prefix, pkg_chrony, self.root_path)
        ret = call_cmd(cmd)
        if ret['ret'] == 0:
            print("-"*20)
        else:
            print("="*20)
        print("Please Wait...")
        time.sleep(5)

    def cur_rpm_info(self):
        """ get RPM list infomation of current system """
        rpm_list = get_rpm_list(tg_path=self.root_path)
        total = len(rpm_list)
        cnt = 0
        for rpm in rpm_list:
            rname = get_rpm_info("name", rpm, tg_path=self.root_path).strip()
            rversion = get_rpm_info("version", rpm, tg_path=self.root_path).strip()
            rrelease = get_rpm_info("release", rpm, tg_path=self.root_path).strip()
            if rname and rversion and rrelease:
                if rname and rversion and rrelease:
                    self.cur_rpm_dict[rname] = {"ver": rversion, "rel": rrelease}
                    cnt += 1
                    progress(100 * cnt / total)
            else:
                continue
        print(_("\nfinished!"))

    def dict_set_compare(self):
        """ Compare RPM list information with set """
        set_cur_keys = set(self.cur_rpm_dict.keys())
        set_ori_keys = set(self.ori_rpm_dict.keys())
        self.set_custom_add = set_cur_keys - set_ori_keys
        self.set_custom_del = set_ori_keys - set_cur_keys
        self.set_common = set_cur_keys & set_ori_keys

        self.set_common_compare()
        self.set_del_compare()
        self.set_add_compare()

    def set_common_compare(self):
        """ compare of the same parts """
        list_common_changed = []
        list_changed_to_upgrade = []
        list_changed_to_downgrade = []
        list_changed_to_reinstall = []
        for ckey in self.set_common:
            ori_ver = self.ori_rpm_dict.get(ckey).get("ver")
            cur_ver = self.cur_rpm_dict.get(ckey).get("ver")

            ori_rel = self.ori_rpm_dict.get(ckey).get("rel")
            cur_rel = self.cur_rpm_dict.get(ckey).get("rel")
            if (ori_ver == cur_ver) and (ori_rel == cur_rel):
                list_changed_to_reinstall.append(ckey)
            elif (ori_ver + "-" + ori_rel) > (cur_ver + "-" + cur_rel):
                list_changed_to_upgrade.append(ckey)
                with open(CUSTOM_VERSION_DIFFER, "a") as f:
                    f.write("%s-%s-%s > %s-%s-%s\n" % (ckey, ori_ver, ori_rel, ckey, cur_ver, cur_rel))
            elif (ori_ver + "-" + ori_rel) < (cur_ver + "-" + cur_rel):
                list_changed_to_downgrade.append(ckey)
                with open(CUSTOM_VERSION_DIFFER, "a") as f:
                    f.write("%s-%s-%s < %s-%s-%s\n" % (ckey, ori_ver, ori_rel, ckey, cur_ver, cur_rel))
            else:
                list_common_changed.append(ckey)
                with open(CUSTOM_VERSION_DIFFER, "a") as f:
                    f.write("%s-%s-%s ==> %s-%s-%s\n" % (ckey, ori_ver, ori_rel, ckey, cur_ver, cur_rel))

        self.set_common_to_downgrade = set(list_changed_to_downgrade)
        self.set_common_to_upgrade = set(list_changed_to_upgrade)
        self.set_common_to_reinstall = set(list_changed_to_reinstall)
        if self.set_common_to_downgrade or self.set_common_to_upgrade:
            print(_("The RPMS information that was covered has been written to :%s") % CUSTOM_VERSION_DIFFER)
        else:
            print(_("There is no RPM has been upgrade or downgrade."))

    def set_del_compare(self):
        """ RPM that are uninstalled """
        for dkey in self.set_custom_del:
            ori_ver = self.ori_rpm_dict.get(dkey).get("ver")
            ori_rel = self.ori_rpm_dict.get(dkey).get("rel")

            with open(CUSTOM_DEL_LOG, "a") as f:
                f.write("%s-%s-%s\n" % (dkey, ori_ver, ori_rel))
        if self.set_custom_del:
            print(_("The removed RPMS information is written to:%s") % CUSTOM_DEL_LOG)
        else:
            print(_("There is no removed RPM Package."))

    def set_add_compare(self):
        """ custom installed RPMS """
        for akey in self.set_custom_add:
            cur_ver = self.cur_rpm_dict.get(akey).get("ver")
            cur_rel = self.cur_rpm_dict.get(akey).get("rel")

            with open(CUSTOM_ADD_LOG, "a") as f:
                f.write("%s-%s-%s\n" % (akey, cur_ver, cur_rel))
        if self.set_custom_add:
            print(_("The custom installed RPMS information has been writtem to:%s") % CUSTOM_ADD_LOG)
        else:
            print(_("There is no custom installed RPM package."))

    def handle_del(self, a_tag=""):
        """ Interaction determines whether to uninstall """
        report(CUSTOM_DEL_LOG, "del")
        if a_tag == "hd_all":
            self.install()
        else:
            while True:
                hdel = input(_("Are you sure to install these RPMS？(yes/no)\n> "))
                quit_judge(hdel)
                if yes_or_no(hdel) == 0:
                    break
                elif yes_or_no(hdel) == 1:
                    install_all_opt = input(_("Install all of these or not?(yes/no)\n> "))
                    quit_judge(install_all_opt)
                    if yes_or_no(install_all_opt) == 0:
                        pkg_list = input(_("Please type in the name of RPMS you want to install，separate them with[,] or [:]:\n> "))
                        quit_judge(pkg_list)
                        self.install(pkg_list=pkg_list.replace(",", " "))
                    elif yes_or_no(install_all_opt) == 1:
                        self.install()
                    else:
                        print(_("Invalid option！"))
                    break
                else:
                    continue

    def yum_upgrade(self, pkg_list_to_upgrade=""):
        """ upgrade RPMS """
        pkg_str = ""
        if not pkg_list_to_upgrade:
            for key in self.set_common_to_upgrade:
                pkg_str = pkg_str + key + " "
            if pkg_str:
                os.system("dnf upgrade %s -y --installroot=%s" % (pkg_str, self.root_path))
        else:
            os.system("dnf upgrade %s -y --installroot=%s" % (pkg_list_to_upgrade, self.root_path))

    def yum_downgrade(self, pkg_list_to_downgrade=""):
        """ downgrade RPMS """
        pkg_str = ""
        if not pkg_list_to_downgrade:
            for key in self.set_common_to_downgrade:
                pkg_str = pkg_str + key + " "
            if pkg_str:
                os.system("dnf downgrade %s -y --installroot=%s" % (pkg_str, self.root_path))
        else:
            os.system("dnf downgrade %s -y --installroot=%s" % (pkg_list_to_downgrade, self.root_path))

    def yum_reinstall(self):
        """ reinstall RPMS """
        pkg_str = ""
        for key in self.set_common_to_reinstall:
            if key not in ["filesystem", "setup", "ipa-common", "python2-ipaclent", "ipa-client-common", "python2-ipalib", "ipa-client", "kmod-kvdo", "kylin-repair", "kernel-core", "kernel", "kernel-devel", "kernel-headers", "kernel-tools"]:
                pkg_str = pkg_str + key + " "
        if pkg_str:
            os.system("dnf reinstall %s -y --installroot=%s" % (pkg_str, self.root_path))

    def handle_changed(self, a_tag=""):
        """ handle changed RPMS """
        report(CUSTOM_VERSION_DIFFER, "changed")
        if a_tag == "hd_all":
            if self.set_common_to_upgrade:
                self.yum_upgrade()
            if self.set_common_to_downgrade:
                self.yum_downgrade()
        else:
            while True:
                hchange = input(_("Are you sure to reinstall these RPMS?(yes/no)\n> "))
                quit_judge(hchange)
                if yes_or_no(hchange) == 0:
                    break
                elif yes_or_no(hchange) == 1:
                    reinst_all_opt = input(_("Reinstall All of these RPMS？(yes/no)\n> "))
                    quit_judge(reinst_all_opt)
                    if yes_or_no(reinst_all_opt) == 0:
                        pkg_str_to_upgrade = ""
                        pkg_str_to_downgrade = ""
                        pkg_str_not_in = ""
                        pkg_list_str = input(_("Please type in the name of RPMS you want to reinstall，separate them with[,] or [:]:\n> "))
                        quit_judge(pkg_list_str)
                        pkg_list = pkg_list_str.replace(",", " ").split()
                        for pkg in pkg_list:
                            if pkg in self.set_common_to_downgrade:
                                pkg_str_to_downgrade = pkg_str_to_downgrade + pkg + " "
                            elif pkg in self.set_common_to_upgrade:
                                pkg_str_to_upgrade = pkg_str_to_upgrade + pkg + " "
                            else:
                                pkg_str_not_in = pkg_str_not_in + pkg + " "
                        if pkg_str_to_upgrade:
                            self.yum_upgrade(pkg_str_to_upgrade)
                        if pkg_str_to_downgrade:
                            self.yum_downgrade(pkg_str_to_downgrade)
                        if pkg_str_not_in:
                            self.install(pkg_str_not_in)

                    elif yes_or_no(reinst_all_opt) == 1:
                        if self.set_common_to_upgrade:
                            self.yum_upgrade()
                        if self.set_common_to_downgrade:
                            self.yum_downgrade()
                    else:
                        print(_("Invalid option！"))
                    break
                else:
                    continue

    def handl_add(self):
        """ handle custom installed RPMS """
        report(CUSTOM_ADD_LOG, "add")
        while True:
            hadd = input(_("Do you want to uninstall these RPMS?(yes/no)\n> "))
            quit_judge(hadd)
            if yes_or_no(hadd) == 0:
                break
            elif yes_or_no(hadd) == 1:
                remove_all_opt = input(_("Uninstall all of these.(yes/no)\n> "))
                quit_judge(remove_all_opt)
                if yes_or_no(remove_all_opt) == 0:
                    pkg_list = input(_("Please type in the name of RPMS you want to uninstall，separate them with[,] or [:]:\n> "))
                    quit_judge(pkg_list)
                    self.uninstall(pkg_list=pkg_list.replace(",", " "))
                elif yes_or_no(remove_all_opt) == 1:
                    self.uninstall()
                else:
                    print(_("Invalid option!"))
                break
            else:
                continue

    def hadle_rest(self):
        """ reinstall """
        self.yum_reinstall()

    def install(self, pkg_list=""):
        """ install RPMS """
        pkg_str = ""
        if not pkg_list:
            for key in self.set_custom_del:
                pkg_str = pkg_str + key + " "
            os.system("dnf install %s -y --installroot=%s" % (pkg_str, self.root_path))
        else:
            try:
                os.system("dnf install %s -y --installroot=%s" % (pkg_list, self.root_path))
            except Exception as e:
                print(e)

    def uninstall(self, pkg_list=""):
        """ uninstall RPMS """
        pkg_str = ""
        if not pkg_list:
            for key in self.set_custom_add:
                pkg_str = pkg_str + key + " "
            os.system("dnf remove %s -y --installroot=%s" % (pkg_str, self.root_path))
        else:
            try:
                os.system("dnf remove %s -y --installroot=%s" % (pkg_list, self.root_path))
            except Exception as e:
                print(e)
